/*
 *****************************************************************************
 *                                                                           *
 *              IMPINJ CONFIDENTIAL AND PROPRIETARY                          *
 *                                                                           *
 * This source code is the sole property of Impinj, Inc. Reproduction or     *
 * utilization of this source code in whole or in part is forbidden without  *
 * the prior written consent of Impinj, Inc.                                 *
 *                                                                           *
 * (c) Copyright Impinj, Inc. 2013-2015. All rights reserved.                *
 *                                                                           *
 *****************************************************************************/
#ifndef __IPJ_UTIL_H
#define __IPJ_UTIL_H

#include "iri.h"

/* Macro for helping handle errors */
#define IPJ_UTIL_PRINT_ERROR(e, msg) \
printf("ERROR: %s Failed - [ERROR CODE: 0x%X]\n\n", msg, e); \

#define IPJ_UTIL_RETURN_ON_ERROR(e, msg) \
if (e)                                                \
{                                                     \
    IPJ_UTIL_PRINT_ERROR(e, msg); \
    return e;                                         \
}

/* Macro for helping check for COM Port input */
#define IPJ_UTIL_CHECK_USER_INPUT_FOR_COM_PORT_RETURN_ON_ERROR() \
if (argc != 2) \
{ \
    printf("\n\nUsage:\t%s COMx\n\nwhere x is a COM port number\n\n", argv[0]); \
    return -1; \
} \

struct ipj_handler
{
    ipj_handler_type type;
    void* handler;
};

ipj_error ipj_util_setup(ipj_iri_device* iri_device, char* reader_identifier);
ipj_error ipj_util_cleanup(ipj_iri_device* iri_device);

void ipj_util_print_epc(uint16_t* epc, int len, bool little_endian);
void ipj_util_print_divider(char symbol, int width);

ipj_error ipj_util_register_handlers(ipj_iri_device* iri_device);
ipj_error ipj_util_perform_inventory(
        ipj_iri_device* iri_device,
        uint32_t timeout_ms);

ipj_error ipj_util_test_command(
        ipj_iri_device* iri_device,
        ipj_test_id test_id,
        uint32_t param0,
        uint32_t param1,
        uint32_t param2,
        uint32_t param3);

ipj_error ipj_util_report_handler(
        ipj_iri_device* iri_device,
        ipj_report_id report_id,
        void* report);
ipj_error ipj_util_tag_operation_report_handler(
        ipj_iri_device* iri_device,
        ipj_tag_operation_report* tag_operation_report);
ipj_error ipj_util_stop_report_handler(
        ipj_iri_device* iri_device,
        ipj_stop_report* stop_report);
ipj_error ipj_util_gpio_report_handler(
        ipj_iri_device* iri_device,
        ipj_gpio_report* gpio_report);
void ipj_util_diagnostic_handler(ipj_iri_device* iri_device, ipj_error error);
ipj_error ipj_util_error_report_handler(
        ipj_iri_device* iri_device,
        ipj_error_report* error_report);
ipj_error ipj_util_status_report_handler(
        ipj_iri_device* iri_device,
        ipj_status_report* status_report);
ipj_error ipj_util_test_report_handler(
        ipj_iri_device* iri_device,
        ipj_test_report* test_report);

#endif
